@@ -14,11 +14,20 @@ sub new {
         }
         $buffer .= $line."\n";
     }
-
     my $ctx = Digest::SHA->new('256');
     my $digest = uc $ctx->add($buffer)->hexdigest;
 
+    my $parser = new MIME::Parser;
+    $parser->output_to_core(1);
+    $header = $parser->parse_data($content)->head;
+    my $msgid = $header->get('Message-ID');
+    chomp $msgid;
+    $msgid =~ s/[<>]//g;
+    $msgid =~ s/\@//g;
+    $msgid = 'NOMSGID' unless $msgid;
+
     my $self = {
+        msgid     => $msgid,
         digest    => $digest,
         content   => $content,
         mode      => $mode,
@@ -35,13 +44,23 @@ sub save
     my $suffix = $sfx or 'eml';
     $suffix =~ s/^\.//;
 
-    my $tmpname = $path.'/'.$self->{digest};
+    my $tmpname = $path.'/'."$self->{msgid}".':'.$self->{digest};
     $tmpname =~ s/\/\//\//g;
     $tmpname .= '.tmp';
 
-    open my $fh, '>',$tmpname or die "Cannot write $tmpname: $!\n";
-    print $fh $self->{content};
-    close $fh or die "Cannot close $tmpname: $!\n";
+    eval {
+        open my $fh, '>',$tmpname or die "Cannot write $tmpname: $!\n";
+        print $fh $self->{content};
+        close $fh or die "Cannot close $tmpname: $!\n";
+    };
+    if ($@) {   # file name to long?
+        $tmpname = $path.'/'.$self->{digest};
+        $tmpname =~ s/\/\//\//g;
+        $tmpname .= '.tmp';
+        open my $fh, '>',$tmpname or die "Cannot write $tmpname: $!\n";
+        print $fh $self->{content};
+        close $fh or die "Cannot close $tmpname: $!\n";
+    }
 
     (my $emlname = $tmpname) =~ s/\.tmp$/.$suffix/;
     chmod oct $self->{mode}, $tmpname;
